@@ -1,11 +1,11 @@
-// Copyright 2011-2020 David Robillard <d@drobilla.net>
+// Copyright 2011-2023 David Robillard <d@drobilla.net>
 // SPDX-License-Identifier: ISC
 
-#include "byte_source.h"
 #include "reader.h"
 #include "serd_internal.h"
 #include "stack.h"
 #include "string_utils.h"
+#include "try.h"
 #include "uri_utils.h"
 
 #include "serd/serd.h"
@@ -14,7 +14,6 @@
 #include <stdbool.h>
 #include <stdint.h>
 #include <stdio.h>
-#include <stdlib.h>
 #include <string.h>
 
 #if defined(__clang__) && __clang_major__ >= 10
@@ -27,13 +26,6 @@ _Pragma("clang diagnostic ignored \"-Wmissing-declarations\"")
 #  define SERD_FALLTHROUGH
 #endif
 
-#define TRY(st, exp)      \
-  do {                    \
-    if (((st) = (exp))) { \
-      return (st);        \
-    }                     \
-  } while (0)
-
 static bool
 fancy_syntax(const SerdReader* const reader)
 {
@@ -54,7 +46,7 @@ read_HEX(SerdReader* const reader)
     return (uint8_t)eat_byte_safe(reader, c);
   }
 
-  r_err(reader, SERD_ERR_BAD_SYNTAX, "invalid hexadecimal digit `%c'\n", c);
+  r_err(reader, SERD_ERR_BAD_SYNTAX, "invalid hexadecimal digit '%c'\n", c);
   return 0;
 }
 
@@ -75,19 +67,20 @@ read_UCHAR(SerdReader* const reader, const Ref dest, uint32_t* const char_code)
     return SERD_ERR_BAD_SYNTAX;
   }
 
-  eat_byte_safe(reader, b);
+  skip_byte(reader, b);
 
-  uint8_t buf[9] = {0, 0, 0, 0, 0, 0, 0, 0, 0};
-  for (unsigned i = 0; i < length; ++i) {
+  // Read character code point in hex
+  uint8_t  buf[9] = {0, 0, 0, 0, 0, 0, 0, 0, 0};
+  uint32_t code   = 0U;
+  for (unsigned i = 0U; i < length; ++i) {
     if (!(buf[i] = read_HEX(reader))) {
       return SERD_ERR_BAD_SYNTAX;
     }
+
+    code = (code << (i ? 4U : 0U)) | hex_digit_value(buf[i]);
   }
 
-  char*          endptr = NULL;
-  const uint32_t code   = (uint32_t)strtoul((const char*)buf, &endptr, 16);
-  assert(endptr == (char*)buf + length);
-
+  // Determine the encoded size from the code point
   unsigned size = 0;
   if (code < 0x00000080) {
     size = 1;
@@ -112,17 +105,17 @@ read_UCHAR(SerdReader* const reader, const Ref dest, uint32_t* const char_code)
   uint32_t c = code;
   switch (size) {
   case 4:
-    buf[3] = (uint8_t)(0x80u | (c & 0x3Fu));
+    buf[3] = (uint8_t)(0x80U | (c & 0x3FU));
     c >>= 6;
     c |= (16 << 12); // set bit 4
     SERD_FALLTHROUGH;
   case 3:
-    buf[2] = (uint8_t)(0x80u | (c & 0x3Fu));
+    buf[2] = (uint8_t)(0x80U | (c & 0x3FU));
     c >>= 6;
     c |= (32 << 6); // set bit 5
     SERD_FALLTHROUGH;
   case 2:
-    buf[1] = (uint8_t)(0x80u | (c & 0x3Fu));
+    buf[1] = (uint8_t)(0x80U | (c & 0x3FU));
     c >>= 6;
     c |= 0xC0; // set bits 6 and 7
     SERD_FALLTHROUGH;
@@ -142,35 +135,25 @@ read_UCHAR(SerdReader* const reader, const Ref dest, uint32_t* const char_code)
 static SerdStatus
 read_ECHAR(SerdReader* const reader, const Ref dest, SerdNodeFlags* const flags)
 {
-  const int c = peek_byte(reader);
+  SerdStatus st = SERD_SUCCESS;
+  const int  c  = peek_byte(reader);
   switch (c) {
   case 't':
-    eat_byte_safe(reader, 't');
-    push_byte(reader, dest, '\t');
-    return SERD_SUCCESS;
+    return (st = skip_byte(reader, 't')) ? st : push_byte(reader, dest, '\t');
   case 'b':
-    eat_byte_safe(reader, 'b');
-    push_byte(reader, dest, '\b');
-    return SERD_SUCCESS;
+    return (st = skip_byte(reader, 'b')) ? st : push_byte(reader, dest, '\b');
   case 'n':
     *flags |= SERD_HAS_NEWLINE;
-    eat_byte_safe(reader, 'n');
-    push_byte(reader, dest, '\n');
-    return SERD_SUCCESS;
+    return (st = skip_byte(reader, 'n')) ? st : push_byte(reader, dest, '\n');
   case 'r':
     *flags |= SERD_HAS_NEWLINE;
-    eat_byte_safe(reader, 'r');
-    push_byte(reader, dest, '\r');
-    return SERD_SUCCESS;
+    return (st = skip_byte(reader, 'r')) ? st : push_byte(reader, dest, '\r');
   case 'f':
-    eat_byte_safe(reader, 'f');
-    push_byte(reader, dest, '\f');
-    return SERD_SUCCESS;
+    return (st = skip_byte(reader, 'f')) ? st : push_byte(reader, dest, '\f');
   case '\\':
   case '"':
   case '\'':
-    push_byte(reader, dest, eat_byte_safe(reader, c));
-    return SERD_SUCCESS;
+    return push_byte(reader, dest, eat_byte_safe(reader, c));
   default:
     return SERD_ERR_BAD_SYNTAX;
   }
@@ -181,7 +164,7 @@ bad_char(SerdReader* const reader, const char* const fmt, const uint8_t c)
 {
   // Skip bytes until the next start byte
   for (int b = peek_byte(reader); b != EOF && ((uint8_t)b & 0x80);) {
-    eat_byte_safe(reader, b);
+    skip_byte(reader, b);
     b = peek_byte(reader);
   }
 
@@ -207,8 +190,7 @@ read_utf8_bytes(SerdReader* const reader,
       return bad_char(reader, "invalid UTF-8 continuation 0x%X\n", (uint8_t)b);
     }
 
-    eat_byte_safe(reader, b);
-    bytes[i] = (uint8_t)b;
+    bytes[i] = (uint8_t)eat_byte_safe(reader, b);
   }
 
   return SERD_SUCCESS;
@@ -279,10 +261,11 @@ read_character(SerdReader* const    reader,
 static void
 read_comment(SerdReader* const reader)
 {
-  eat_byte_safe(reader, '#');
+  skip_byte(reader, '#');
+
   int c = 0;
   while (((c = peek_byte(reader)) != 0xA) && c != 0xD && c != EOF && c) {
-    eat_byte_safe(reader, c);
+    skip_byte(reader, c);
   }
 }
 
@@ -296,7 +279,7 @@ read_ws(SerdReader* const reader)
   case 0xA:
   case 0xD:
   case 0x20:
-    eat_byte_safe(reader, c);
+    skip_byte(reader, c);
     return true;
   case '#':
     read_comment(reader);
@@ -326,13 +309,28 @@ static bool
 eat_delim(SerdReader* const reader, const uint8_t delim)
 {
   if (peek_delim(reader, delim)) {
-    eat_byte_safe(reader, delim);
+    skip_byte(reader, delim);
     return read_ws_star(reader);
   }
 
   return false;
 }
 
+static SerdStatus
+read_string_escape(SerdReader* const    reader,
+                   const Ref            ref,
+                   SerdNodeFlags* const flags)
+{
+  SerdStatus st   = SERD_SUCCESS;
+  uint32_t   code = 0;
+  if ((st = read_ECHAR(reader, ref, flags)) &&
+      (st = read_UCHAR(reader, ref, &code))) {
+    return r_err(reader, st, "invalid escape '\\%c'\n", peek_byte(reader));
+  }
+
+  return st;
+}
+
 // STRING_LITERAL_LONG_QUOTE and STRING_LITERAL_LONG_SINGLE_QUOTE
 // Initial triple quotes are already eaten by caller
 static SerdStatus
@@ -346,25 +344,27 @@ read_STRING_LITERAL_LONG(SerdReader* const    reader,
   while (!(st && reader->strict)) {
     const int c = peek_byte(reader);
     if (c == '\\') {
-      eat_byte_safe(reader, c);
-      uint32_t code = 0;
-      if ((st = read_ECHAR(reader, ref, flags)) &&
-          (st = read_UCHAR(reader, ref, &code))) {
-        return r_err(reader, st, "invalid escape `\\%c'\n", peek_byte(reader));
-      }
+      skip_byte(reader, c);
+      st = read_string_escape(reader, ref, flags);
     } else if (c == q) {
-      eat_byte_safe(reader, q);
+      skip_byte(reader, q);
       const int q2 = eat_byte_safe(reader, peek_byte(reader));
       const int q3 = peek_byte(reader);
       if (q2 == q && q3 == q) { // End of string
-        eat_byte_safe(reader, q3);
+        skip_byte(reader, q3);
         break;
       }
-      *flags |= SERD_HAS_QUOTE;
-      push_byte(reader, ref, c);
-      st = read_character(reader, ref, flags, (uint8_t)q2);
+
+      if (q2 == '\\') {
+        push_byte(reader, ref, c);
+        st = read_string_escape(reader, ref, flags);
+      } else {
+        *flags |= SERD_HAS_QUOTE;
+        push_byte(reader, ref, c);
+        st = read_character(reader, ref, flags, (uint8_t)q2);
+      }
     } else if (c == EOF) {
-      return r_err(reader, SERD_ERR_BAD_SYNTAX, "end of file in long string\n");
+      st = r_err(reader, SERD_ERR_BAD_SYNTAX, "end of file in long string\n");
     } else {
       st =
         read_character(reader, ref, flags, (uint8_t)eat_byte_safe(reader, c));
@@ -384,9 +384,8 @@ read_STRING_LITERAL(SerdReader* const    reader,
 {
   SerdStatus st = SERD_SUCCESS;
 
-  while (!(st && reader->strict)) {
-    const int c    = peek_byte(reader);
-    uint32_t  code = 0;
+  while (!st || !reader->strict) {
+    const int c = peek_byte(reader);
     switch (c) {
     case EOF:
       return r_err(
@@ -395,25 +394,20 @@ read_STRING_LITERAL(SerdReader* const    reader,
     case '\r':
       return r_err(reader, SERD_ERR_BAD_SYNTAX, "line end in short string\n");
     case '\\':
-      eat_byte_safe(reader, c);
-      if ((st = read_ECHAR(reader, ref, flags)) &&
-          (st = read_UCHAR(reader, ref, &code))) {
-        return r_err(reader, st, "invalid escape `\\%c'\n", peek_byte(reader));
-      }
+      skip_byte(reader, c);
+      TRY(st, read_string_escape(reader, ref, flags));
       break;
     default:
       if (c == q) {
-        eat_byte_check(reader, q);
-        return SERD_SUCCESS;
-      } else {
-        st =
-          read_character(reader, ref, flags, (uint8_t)eat_byte_safe(reader, c));
+        return skip_byte(reader, c);
       }
+
+      st =
+        read_character(reader, ref, flags, (uint8_t)eat_byte_safe(reader, c));
     }
   }
 
-  return st ? st
-            : (eat_byte_check(reader, q) ? SERD_SUCCESS : SERD_ERR_BAD_SYNTAX);
+  return eat_byte_check(reader, q) ? SERD_SUCCESS : SERD_ERR_BAD_SYNTAX;
 }
 
 static SerdStatus
@@ -421,9 +415,7 @@ read_String(SerdReader* const    reader,
             const Ref            node,
             SerdNodeFlags* const flags)
 {
-  const int q1 = peek_byte(reader);
-  eat_byte_safe(reader, q1);
-
+  const int q1 = eat_byte_safe(reader, peek_byte(reader));
   const int q2 = peek_byte(reader);
   if (q2 == EOF) {
     return r_err(reader, SERD_ERR_BAD_SYNTAX, "unexpected end of file\n");
@@ -433,7 +425,7 @@ read_String(SerdReader* const    reader,
     return read_STRING_LITERAL(reader, node, flags, (uint8_t)q1);
   }
 
-  eat_byte_safe(reader, q2);
+  skip_byte(reader, q2);
   const int q3 = peek_byte(reader);
   if (q3 == EOF) {
     return r_err(reader, SERD_ERR_BAD_SYNTAX, "unexpected end of file\n");
@@ -448,7 +440,7 @@ read_String(SerdReader* const    reader,
       reader, SERD_ERR_BAD_SYNTAX, "syntax does not support long literals\n");
   }
 
-  eat_byte_safe(reader, q3);
+  skip_byte(reader, q3);
   return read_STRING_LITERAL_LONG(reader, node, flags, (uint8_t)q1);
 }
 
@@ -469,20 +461,26 @@ read_PN_CHARS_BASE(SerdReader* const reader, const Ref dest)
   uint32_t   code = 0;
   const int  c    = peek_byte(reader);
   SerdStatus st   = SERD_SUCCESS;
+
   if (is_alpha(c)) {
-    push_byte(reader, dest, eat_byte_safe(reader, c));
-  } else if (c == EOF || !(c & 0x80)) {
+    return push_byte(reader, dest, eat_byte_safe(reader, c));
+  }
+
+  if (c == EOF || !(c & 0x80)) {
     return SERD_FAILURE;
-  } else if ((st = read_utf8_code(
-                reader, dest, &code, (uint8_t)eat_byte_safe(reader, c)))) {
-    return st;
-  } else if (!is_PN_CHARS_BASE(code)) {
+  }
+
+  skip_byte(reader, c);
+  read_utf8_code(reader, dest, &code, (uint8_t)c);
+
+  if (!is_PN_CHARS_BASE(code)) {
     r_err(
       reader, SERD_ERR_BAD_SYNTAX, "invalid character U+%04X in name\n", code);
     if (reader->strict) {
       return SERD_ERR_BAD_SYNTAX;
     }
   }
+
   return st;
 }
 
@@ -499,17 +497,23 @@ read_PN_CHARS(SerdReader* const reader, const Ref dest)
   uint32_t   code = 0;
   const int  c    = peek_byte(reader);
   SerdStatus st   = SERD_SUCCESS;
+
   if (is_alpha(c) || is_digit(c) || c == '_' || c == '-') {
-    push_byte(reader, dest, eat_byte_safe(reader, c));
-  } else if (c == EOF || !(c & 0x80)) {
+    return push_byte(reader, dest, eat_byte_safe(reader, c));
+  }
+
+  if (c == EOF || !(c & 0x80)) {
     return SERD_FAILURE;
-  } else if ((st = read_utf8_code(
-                reader, dest, &code, (uint8_t)eat_byte_safe(reader, c)))) {
-    return st;
-  } else if (!is_PN_CHARS(code)) {
+  }
+
+  skip_byte(reader, c);
+  TRY(st, read_utf8_code(reader, dest, &code, (uint8_t)c));
+
+  if (!is_PN_CHARS(code)) {
     return r_err(
       reader, SERD_ERR_BAD_SYNTAX, "invalid character U+%04X in name\n", code);
   }
+
   return st;
 }
 
@@ -530,7 +534,7 @@ read_PERCENT(SerdReader* const reader, const Ref dest)
 static SerdStatus
 read_PN_LOCAL_ESC(SerdReader* const reader, const Ref dest)
 {
-  eat_byte_safe(reader, '\\');
+  skip_byte(reader, '\\');
 
   const int c = peek_byte(reader);
   switch (c) {
@@ -601,8 +605,10 @@ read_PN_LOCAL(SerdReader* const reader, const Ref dest, bool* const ate_dot)
   default:
     if ((st = read_PLX(reader, dest)) > SERD_FAILURE) {
       return r_err(reader, st, "bad escape\n");
-    } else if (st != SERD_SUCCESS && read_PN_CHARS_BASE(reader, dest)) {
-      return SERD_FAILURE;
+    }
+
+    if (st != SERD_SUCCESS && (st = read_PN_CHARS_BASE(reader, dest))) {
+      return st;
     }
   }
 
@@ -643,7 +649,7 @@ read_PN_PREFIX_tail(SerdReader* const reader, const Ref dest)
 
   const SerdNode* const n = deref(reader, dest);
   if (n->buf[n->n_bytes - 1] == '.' && read_PN_CHARS(reader, dest)) {
-    return r_err(reader, SERD_ERR_BAD_SYNTAX, "prefix ends with `.'\n");
+    return r_err(reader, SERD_ERR_BAD_SYNTAX, "prefix ends with '.'\n");
   }
 
   return SERD_SUCCESS;
@@ -652,11 +658,9 @@ read_PN_PREFIX_tail(SerdReader* const reader, const Ref dest)
 static SerdStatus
 read_PN_PREFIX(SerdReader* const reader, const Ref dest)
 {
-  if (!read_PN_CHARS_BASE(reader, dest)) {
-    return read_PN_PREFIX_tail(reader, dest);
-  }
+  const SerdStatus st = read_PN_CHARS_BASE(reader, dest);
 
-  return SERD_FAILURE;
+  return st ? st : read_PN_PREFIX_tail(reader, dest);
 }
 
 static SerdStatus
@@ -664,7 +668,7 @@ read_LANGTAG(SerdReader* const reader, Ref* const dest)
 {
   int c = peek_byte(reader);
   if (!is_alpha(c)) {
-    return r_err(reader, SERD_ERR_BAD_SYNTAX, "unexpected `%c'\n", c);
+    return r_err(reader, SERD_ERR_BAD_SYNTAX, "unexpected '%c'\n", c);
   }
 
   *dest = push_node(reader, SERD_LITERAL, "", 0);
@@ -690,7 +694,7 @@ read_IRIREF_scheme(SerdReader* const reader, const Ref dest)
 {
   int c = peek_byte(reader);
   if (!is_alpha(c)) {
-    return r_err(reader, SERD_ERR_BAD_SYNTAX, "bad IRI scheme start `%c'\n", c);
+    return r_err(reader, SERD_ERR_BAD_SYNTAX, "bad IRI scheme start '%c'\n", c);
   }
 
   while ((c = peek_byte(reader)) != EOF) {
@@ -738,7 +742,7 @@ read_IRIREF(SerdReader* const reader, Ref* const dest)
     case '<':
       *dest = pop_node(reader, *dest);
       return r_err(
-        reader, SERD_ERR_BAD_SYNTAX, "invalid IRI character `%c'\n", c);
+        reader, SERD_ERR_BAD_SYNTAX, "invalid IRI character '%c'\n", c);
 
     case '>':
       return SERD_SUCCESS;
@@ -771,7 +775,7 @@ read_IRIREF(SerdReader* const reader, Ref* const dest)
     case '}':
       *dest = pop_node(reader, *dest);
       return r_err(
-        reader, SERD_ERR_BAD_SYNTAX, "invalid IRI character `%c'\n", c);
+        reader, SERD_ERR_BAD_SYNTAX, "invalid IRI character '%c'\n", c);
 
     default:
       if (c <= 0x20) {
@@ -807,8 +811,8 @@ read_PrefixedName(SerdReader* const reader,
                   bool* const       ate_dot)
 {
   SerdStatus st = SERD_SUCCESS;
-  if (read_prefix && ((st = read_PN_PREFIX(reader, dest)) > SERD_FAILURE)) {
-    return st;
+  if (read_prefix) {
+    TRY_FAILING(st, read_PN_PREFIX(reader, dest));
   }
 
   if (peek_byte(reader) != ':') {
@@ -817,9 +821,8 @@ read_PrefixedName(SerdReader* const reader,
 
   push_byte(reader, dest, eat_byte_safe(reader, ':'));
 
-  st = read_PN_LOCAL(reader, dest, ate_dot);
-
-  return (st > SERD_FAILURE) ? st : SERD_SUCCESS;
+  TRY_FAILING(st, read_PN_LOCAL(reader, dest, ate_dot));
+  return SERD_SUCCESS;
 }
 
 static SerdStatus
@@ -858,7 +861,7 @@ read_number(SerdReader* const reader,
   }
   if ((c = peek_byte(reader)) == '.') {
     has_decimal = true;
-    // decimal case 2 (e.g. '.0' or `-.0' or `+.0')
+    // decimal case 2 (e.g. '.0' or '-.0' or '+.0')
     push_byte(reader, *dest, eat_byte_safe(reader, c));
     TRY(st, read_0_9(reader, *dest, true));
   } else {
@@ -868,7 +871,7 @@ read_number(SerdReader* const reader,
       has_decimal = true;
 
       // Annoyingly, dot can be end of statement, so tentatively eat
-      eat_byte_safe(reader, c);
+      skip_byte(reader, c);
       c = peek_byte(reader);
       if (!is_digit(c) && c != 'e' && c != 'E') {
         *ate_dot = true;     // Force caller to deal with stupid grammar
@@ -934,22 +937,25 @@ read_literal(SerdReader* const    reader,
 
   switch (peek_byte(reader)) {
   case '@':
-    eat_byte_safe(reader, '@');
+    skip_byte(reader, '@');
     if ((st = read_LANGTAG(reader, lang))) {
       *datatype = pop_node(reader, *datatype);
       *lang     = pop_node(reader, *lang);
       *dest     = pop_node(reader, *dest);
-      return r_err(reader, st, "bad literal\n");
+      return r_err(reader, st, "bad language tag\n");
     }
     break;
   case '^':
-    eat_byte_safe(reader, '^');
-    eat_byte_check(reader, '^');
+    skip_byte(reader, '^');
+    if (!eat_byte_check(reader, '^')) {
+      return r_err(reader, SERD_ERR_BAD_SYNTAX, "expected '^'\n");
+    }
+
     if ((st = read_iri(reader, datatype, ate_dot))) {
       *datatype = pop_node(reader, *datatype);
       *lang     = pop_node(reader, *lang);
       *dest     = pop_node(reader, *dest);
-      return r_err(reader, st, "bad literal\n");
+      return r_err(reader, st, "bad datatype\n");
     }
     break;
   }
@@ -980,10 +986,11 @@ read_verb(SerdReader* const reader, Ref* const dest)
     return SERD_SUCCESS;
   }
 
-  if (st > SERD_FAILURE || read_PrefixedName(reader, *dest, false, &ate_dot) ||
-      ate_dot) {
+  if (st > SERD_FAILURE ||
+      (st = read_PrefixedName(reader, *dest, false, &ate_dot)) || ate_dot) {
     *dest = pop_node(reader, *dest);
-    return r_err(reader, SERD_ERR_BAD_SYNTAX, "bad verb\n");
+    st    = st > SERD_FAILURE ? st : SERD_ERR_BAD_SYNTAX;
+    return r_err(reader, st, "bad verb\n");
   }
 
   return SERD_SUCCESS;
@@ -994,8 +1001,10 @@ read_BLANK_NODE_LABEL(SerdReader* const reader,
                       Ref* const        dest,
                       bool* const       ate_dot)
 {
-  eat_byte_safe(reader, '_');
-  eat_byte_check(reader, ':');
+  skip_byte(reader, '_');
+  if (!eat_byte_check(reader, ':')) {
+    return SERD_ERR_BAD_SYNTAX;
+  }
 
   const Ref ref = *dest =
     push_node(reader,
@@ -1036,7 +1045,7 @@ read_BLANK_NODE_LABEL(SerdReader* const reader,
         *dest = pop_node(reader, *dest);
         return r_err(reader,
                      SERD_ERR_ID_CLASH,
-                     "found both `b' and `B' blank IDs, prefix required\n");
+                     "found both 'b' and 'B' blank IDs, prefix required\n");
       }
     }
   }
@@ -1047,9 +1056,8 @@ read_BLANK_NODE_LABEL(SerdReader* const reader,
 static Ref
 read_blankName(SerdReader* const reader)
 {
-  eat_byte_safe(reader, '=');
-  if (eat_byte_check(reader, '=') != '=') {
-    r_err(reader, SERD_ERR_BAD_SYNTAX, "expected `='\n");
+  if (skip_byte(reader, '=') || eat_byte_check(reader, '=') != '=') {
+    r_err(reader, SERD_ERR_BAD_SYNTAX, "expected '='\n");
     return 0;
   }
 
@@ -1066,13 +1074,15 @@ read_anon(SerdReader* const reader,
           const bool        subject,
           Ref* const        dest)
 {
+  skip_byte(reader, '[');
+
   const SerdStatementFlags old_flags = *ctx.flags;
-  bool                     empty     = false;
-  eat_byte_safe(reader, '[');
-  if ((empty = peek_delim(reader, ']'))) {
-    *ctx.flags |= (subject) ? SERD_EMPTY_S : SERD_EMPTY_O;
+  const bool               empty     = peek_delim(reader, ']');
+
+  if (subject) {
+    *ctx.flags |= empty ? SERD_EMPTY_S : SERD_ANON_S_BEGIN;
   } else {
-    *ctx.flags |= (subject) ? SERD_ANON_S_BEGIN : SERD_ANON_O_BEGIN;
+    *ctx.flags |= empty ? SERD_EMPTY_O : SERD_ANON_O_BEGIN;
     if (peek_delim(reader, '=')) {
       if (!(*dest = read_blankName(reader)) || !eat_delim(reader, ';')) {
         return SERD_ERR_BAD_SYNTAX;
@@ -1084,11 +1094,13 @@ read_anon(SerdReader* const reader,
     *dest = blank_id(reader);
   }
 
+  // Emit statement with this anonymous object first
   SerdStatus st = SERD_SUCCESS;
   if (ctx.subject) {
     TRY(st, emit_statement(reader, ctx, *dest, 0, 0));
   }
 
+  // Switch the subject to the anonymous node and read its description
   ctx.subject = *dest;
   if (!empty) {
     *ctx.flags &= ~(unsigned)SERD_LIST_CONT;
@@ -1097,9 +1109,10 @@ read_anon(SerdReader* const reader,
     }
 
     bool ate_dot_in_list = false;
-    read_predicateObjectList(reader, ctx, &ate_dot_in_list);
+    TRY_FAILING(st, read_predicateObjectList(reader, ctx, &ate_dot_in_list));
+
     if (ate_dot_in_list) {
-      return r_err(reader, SERD_ERR_BAD_SYNTAX, "`.' inside blank\n");
+      return r_err(reader, SERD_ERR_BAD_SYNTAX, "'.' inside blank\n");
     }
 
     read_ws_star(reader);
@@ -1110,8 +1123,9 @@ read_anon(SerdReader* const reader,
     *ctx.flags = old_flags;
   }
 
-  return (eat_byte_check(reader, ']') == ']') ? SERD_SUCCESS
-                                              : SERD_ERR_BAD_SYNTAX;
+  return st > SERD_FAILURE                      ? st
+         : (eat_byte_check(reader, ']') == ']') ? SERD_SUCCESS
+                                                : SERD_ERR_BAD_SYNTAX;
 }
 
 /* If emit is true: recurses, calling statement_sink for every statement
@@ -1130,7 +1144,7 @@ read_object(SerdReader* const  reader,
   const size_t orig_stack_size = reader->stack.size;
 #endif
 
-  SerdStatus ret = SERD_FAILURE;
+  SerdStatus st = SERD_FAILURE;
 
   bool      simple   = (ctx->subject != 0);
   SerdNode* node     = NULL;
@@ -1156,18 +1170,18 @@ read_object(SerdReader* const  reader,
     return r_err(reader, SERD_ERR_BAD_SYNTAX, "expected object\n");
   case '[':
     simple = false;
-    ret    = read_anon(reader, *ctx, false, &o);
+    st     = read_anon(reader, *ctx, false, &o);
     break;
   case '(':
     simple = false;
-    ret    = read_collection(reader, *ctx, &o);
+    st     = read_collection(reader, *ctx, &o);
     break;
   case '_':
-    ret = read_BLANK_NODE_LABEL(reader, &o, ate_dot);
+    st = read_BLANK_NODE_LABEL(reader, &o, ate_dot);
     break;
   case '<':
   case ':':
-    ret = read_iri(reader, &o, ate_dot);
+    st = read_iri(reader, &o, ate_dot);
     break;
   case '+':
   case '-':
@@ -1182,11 +1196,11 @@ read_object(SerdReader* const  reader,
   case '7':
   case '8':
   case '9':
-    ret = read_number(reader, &o, &datatype, ate_dot);
+    st = read_number(reader, &o, &datatype, ate_dot);
     break;
   case '\"':
   case '\'':
-    ret = read_literal(reader, &o, &datatype, &lang, &flags, ate_dot);
+    st = read_literal(reader, &o, &datatype, &lang, &flags, ate_dot);
     break;
   default:
     /* Either a boolean literal, or a qname.  Read the prefix first, and if
@@ -1200,25 +1214,25 @@ read_object(SerdReader* const  reader,
         (node->n_bytes == 5 && !memcmp(node->buf, "false", 5))) {
       node->type = SERD_LITERAL;
       datatype   = push_node(reader, SERD_URI, XSD_BOOLEAN, XSD_BOOLEAN_LEN);
-      ret        = SERD_SUCCESS;
+      st         = SERD_SUCCESS;
     } else if (read_PN_PREFIX_tail(reader, o) > SERD_FAILURE) {
-      ret = SERD_ERR_BAD_SYNTAX;
+      st = SERD_ERR_BAD_SYNTAX;
     } else {
-      if ((ret = read_PrefixedName(reader, o, false, ate_dot))) {
-        ret = ret > SERD_FAILURE ? ret : SERD_ERR_BAD_SYNTAX;
+      if ((st = read_PrefixedName(reader, o, false, ate_dot))) {
+        st = st > SERD_FAILURE ? st : SERD_ERR_BAD_SYNTAX;
         pop_node(reader, o);
-        return r_err(reader, ret, "expected prefixed name\n");
+        return r_err(reader, st, "expected prefixed name\n");
       }
     }
   }
 
-  if (!ret && simple && o) {
+  if (!st && simple && o) {
     deref(reader, o)->flags = flags;
   }
 
-  if (!ret && emit && simple) {
-    ret = emit_statement(reader, *ctx, o, datatype, lang);
-  } else if (!ret && !emit) {
+  if (!st && emit && simple) {
+    st = emit_statement(reader, *ctx, o, datatype, lang);
+  } else if (!st && !emit) {
     ctx->object   = o;
     ctx->datatype = datatype;
     ctx->lang     = lang;
@@ -1231,7 +1245,7 @@ read_object(SerdReader* const  reader,
 #ifndef NDEBUG
   assert(reader->stack.size == orig_stack_size);
 #endif
-  return ret;
+  return st;
 }
 
 static SerdStatus
@@ -1276,7 +1290,7 @@ read_predicateObjectList(SerdReader* const reader,
       case '}':
         return SERD_SUCCESS;
       case ';':
-        eat_byte_safe(reader, c);
+        skip_byte(reader, c);
         ate_semi = true;
       }
     } while (c == ';');
@@ -1312,17 +1326,17 @@ static SerdStatus
 read_collection(SerdReader* const reader, ReadContext ctx, Ref* const dest)
 {
   SerdStatus st = SERD_SUCCESS;
-  eat_byte_safe(reader, '(');
+  skip_byte(reader, '(');
 
   bool end = peek_delim(reader, ')');
 
   *dest = end ? reader->rdf_nil : blank_id(reader);
-  if (ctx.subject) {
-    // subject predicate _:head
+  if (ctx.subject) { // Reading a collection object
     *ctx.flags |= (end ? 0 : SERD_LIST_O_BEGIN);
     TRY(st, emit_statement(reader, ctx, *dest, 0, 0));
+    *ctx.flags &= SERD_LIST_O_BEGIN;
     *ctx.flags |= SERD_LIST_CONT;
-  } else {
+  } else { // Reading a collection subject
     *ctx.flags |= (end ? 0 : SERD_LIST_S_BEGIN);
   }
 
@@ -1379,7 +1393,7 @@ read_subject(SerdReader* const reader,
   bool       ate_dot = false;
   switch ((*s_type = peek_byte(reader))) {
   case '[':
-    read_anon(reader, ctx, true, dest);
+    st = read_anon(reader, ctx, true, dest);
     break;
   case '(':
     st = read_collection(reader, ctx, dest);
@@ -1393,7 +1407,7 @@ read_subject(SerdReader* const reader,
 
   if (ate_dot) {
     pop_node(reader, *dest);
-    return r_err(reader, SERD_ERR_BAD_SYNTAX, "subject ends with `.'\n");
+    return r_err(reader, SERD_ERR_BAD_SYNTAX, "subject ends with '.'\n");
   }
 
   return st;
@@ -1405,7 +1419,7 @@ read_labelOrSubject(SerdReader* const reader, Ref* const dest)
   bool ate_dot = false;
   switch (peek_byte(reader)) {
   case '[':
-    eat_byte_safe(reader, '[');
+    skip_byte(reader, '[');
     read_ws_star(reader);
     if (!eat_byte_check(reader, ']')) {
       return SERD_ERR_BAD_SYNTAX;
@@ -1482,9 +1496,7 @@ read_prefixID(SerdReader* const reader, const bool sparql, const bool token)
 
   read_ws_star(reader);
   Ref name = push_node(reader, SERD_LITERAL, "", 0);
-  if ((st = read_PN_PREFIX(reader, name)) > SERD_FAILURE) {
-    return st;
-  }
+  TRY_FAILING(st, read_PN_PREFIX(reader, name));
 
   if (eat_byte_check(reader, ':') != ':') {
     pop_node(reader, name);
@@ -1515,7 +1527,7 @@ read_directive(SerdReader* const reader)
 {
   const bool sparql = peek_byte(reader) != '@';
   if (!sparql) {
-    eat_byte_safe(reader, '@');
+    skip_byte(reader, '@');
     switch (peek_byte(reader)) {
     case 'B':
     case 'P':
@@ -1554,23 +1566,22 @@ read_wrappedGraph(SerdReader* const reader, ReadContext* const ctx)
       return r_err(reader, SERD_ERR_BAD_SYNTAX, "bad subject\n");
     }
 
-    if (read_triples(reader, *ctx, &ate_dot) && s_type != '[') {
-      return r_err(
-        reader, SERD_ERR_BAD_SYNTAX, "missing predicate object list\n");
+    if ((st = read_triples(reader, *ctx, &ate_dot)) && s_type != '[') {
+      return r_err(reader, st, "bad predicate object list\n");
     }
 
     ctx->subject = pop_node(reader, ctx->subject);
     read_ws_star(reader);
     if (peek_byte(reader) == '.') {
-      eat_byte_safe(reader, '.');
+      skip_byte(reader, '.');
     }
     read_ws_star(reader);
   }
 
-  eat_byte_safe(reader, '}');
+  skip_byte(reader, '}');
   read_ws_star(reader);
   if (peek_byte(reader) == '.') {
-    return r_err(reader, SERD_ERR_BAD_SYNTAX, "graph followed by `.'\n");
+    return r_err(reader, SERD_ERR_BAD_SYNTAX, "graph followed by '.'\n");
   }
 
   return SERD_SUCCESS;
@@ -1605,7 +1616,7 @@ read_n3_statement(SerdReader* const reader)
   read_ws_star(reader);
   switch (peek_byte(reader)) {
   case '\0':
-    eat_byte_safe(reader, '\0');
+    skip_byte(reader, '\0');
     return SERD_FAILURE;
   case EOF:
     return SERD_FAILURE;
@@ -1627,10 +1638,7 @@ read_n3_statement(SerdReader* const reader)
     }
     break;
   default:
-    if ((st = read_subject(reader, ctx, &ctx.subject, &s_type)) >
-        SERD_FAILURE) {
-      return st;
-    }
+    TRY_FAILING(st, read_subject(reader, ctx, &ctx.subject, &s_type));
 
     if (!tokcmp(reader, ctx.subject, "base", 4)) {
       st = read_base(reader, true, false);
@@ -1645,6 +1653,9 @@ read_n3_statement(SerdReader* const reader)
       pop_node(reader, ctx.graph);
       ctx.graph = 0;
       read_ws_star(reader);
+    } else if (!tokcmp(reader, ctx.subject, "true", 4) ||
+               !tokcmp(reader, ctx.subject, "false", 5)) {
+      return r_err(reader, SERD_ERR_BAD_SYNTAX, "expected subject\n");
     } else if (read_ws_star(reader) && peek_byte(reader) == '{') {
       if (s_type == '(' || (s_type == '[' && !*ctx.flags)) {
         return r_err(reader, SERD_ERR_BAD_SYNTAX, "invalid graph name\n");
@@ -1682,12 +1693,17 @@ read_n3_statement(SerdReader* const reader)
   return st;
 }
 
-static void
-skip_until(SerdReader* const reader, const uint8_t byte)
+SerdStatus
+serd_reader_skip_until_byte(SerdReader* const reader, const uint8_t byte)
 {
-  for (int c = 0; (c = peek_byte(reader)) && c != byte;) {
-    eat_byte_safe(reader, c);
+  int c = peek_byte(reader);
+
+  while (c != byte && c != EOF) {
+    skip_byte(reader, c);
+    c = peek_byte(reader);
   }
+
+  return c == EOF ? SERD_FAILURE : SERD_SUCCESS;
 }
 
 SerdStatus
@@ -1699,67 +1715,79 @@ read_turtleTrigDoc(SerdReader* const reader)
       if (reader->strict) {
         return st;
       }
-      skip_until(reader, '\n');
+      serd_reader_skip_until_byte(reader, '\n');
     }
   }
 
   return SERD_SUCCESS;
 }
 
+SerdStatus
+read_nquads_statement(SerdReader* const reader)
+{
+  SerdStatus         st      = SERD_SUCCESS;
+  SerdStatementFlags flags   = 0;
+  ReadContext        ctx     = {0, 0, 0, 0, 0, 0, &flags};
+  bool               ate_dot = false;
+  int                s_type  = 0;
+
+  read_ws_star(reader);
+  if (peek_byte(reader) == EOF) {
+    return SERD_FAILURE;
+  }
+
+  if (peek_byte(reader) == '@') {
+    return r_err(
+      reader, SERD_ERR_BAD_SYNTAX, "syntax does not support directives\n");
+  }
+
+  // subject predicate object
+  if ((st = read_subject(reader, ctx, &ctx.subject, &s_type)) ||
+      !read_ws_star(reader) || (st = read_IRIREF(reader, &ctx.predicate)) ||
+      !read_ws_star(reader) ||
+      (st = read_object(reader, &ctx, false, &ate_dot))) {
+    return st;
+  }
+
+  if (!ate_dot) { // graphLabel?
+    read_ws_star(reader);
+    switch (peek_byte(reader)) {
+    case '.':
+      break;
+    case '_':
+      TRY(st, read_BLANK_NODE_LABEL(reader, &ctx.graph, &ate_dot));
+      break;
+    default:
+      TRY(st, read_IRIREF(reader, &ctx.graph));
+    }
+
+    // Terminating '.'
+    read_ws_star(reader);
+    if (!eat_byte_check(reader, '.')) {
+      return SERD_ERR_BAD_SYNTAX;
+    }
+  }
+
+  TRY(st, emit_statement(reader, ctx, ctx.object, ctx.datatype, ctx.lang));
+
+  pop_node(reader, ctx.graph);
+  pop_node(reader, ctx.lang);
+  pop_node(reader, ctx.datatype);
+  pop_node(reader, ctx.object);
+
+  return SERD_SUCCESS;
+}
+
 SerdStatus
 read_nquadsDoc(SerdReader* const reader)
 {
   SerdStatus st = SERD_SUCCESS;
-  while (!reader->source.eof) {
-    SerdStatementFlags flags   = 0;
-    ReadContext        ctx     = {0, 0, 0, 0, 0, 0, &flags};
-    bool               ate_dot = false;
-    int                s_type  = 0;
-    read_ws_star(reader);
-    if (peek_byte(reader) == EOF) {
-      break;
-    }
 
-    if (peek_byte(reader) == '@') {
-      return r_err(
-        reader, SERD_ERR_BAD_SYNTAX, "syntax does not support directives\n");
-    }
-
-    // subject predicate object
-    if ((st = read_subject(reader, ctx, &ctx.subject, &s_type)) ||
-        !read_ws_star(reader) || (st = read_IRIREF(reader, &ctx.predicate)) ||
-        !read_ws_star(reader) ||
-        (st = read_object(reader, &ctx, false, &ate_dot))) {
-      return st;
-    }
-
-    if (!ate_dot) { // graphLabel?
-      read_ws_star(reader);
-      switch (peek_byte(reader)) {
-      case '.':
-        break;
-      case '_':
-        TRY(st, read_BLANK_NODE_LABEL(reader, &ctx.graph, &ate_dot));
-        break;
-      default:
-        TRY(st, read_IRIREF(reader, &ctx.graph));
-      }
-
-      // Terminating '.'
-      read_ws_star(reader);
-      if (!eat_byte_check(reader, '.')) {
-        return SERD_ERR_BAD_SYNTAX;
-      }
-    }
-
-    TRY(st, emit_statement(reader, ctx, ctx.object, ctx.datatype, ctx.lang));
-
-    pop_node(reader, ctx.graph);
-    pop_node(reader, ctx.lang);
-    pop_node(reader, ctx.datatype);
-    pop_node(reader, ctx.object);
+  while (!reader->source.eof && !st) {
+    st = read_nquads_statement(reader);
   }
-  return SERD_SUCCESS;
+
+  return st;
 }
 
 #if defined(__clang__) && __clang_major__ >= 10
