@@ -1,8 +1,8 @@
-// Copyright 2011-2020 David Robillard <d@drobilla.net>
+// Copyright 2011-2023 David Robillard <d@drobilla.net>
 // SPDX-License-Identifier: ISC
 
-#ifndef SERD_NODE_H
-#define SERD_NODE_H
+#ifndef SERD_SRC_NODE_H
+#define SERD_SRC_NODE_H
 
 #include "serd/serd.h"
 
@@ -14,22 +14,23 @@ struct SerdNodeImpl {
   SerdType      type;    /**< Node type */
 };
 
-static inline char*
-serd_node_buffer(SerdNode* node)
+static inline char* SERD_NONNULL
+serd_node_buffer(SerdNode* SERD_NONNULL node)
 {
   return (char*)(node + 1);
 }
 
-static inline const char*
-serd_node_buffer_c(const SerdNode* node)
+static inline const char* SERD_NONNULL
+serd_node_buffer_c(const SerdNode* SERD_NONNULL node)
 {
   return (const char*)(node + 1);
 }
 
-SerdNode*
+SerdNode* SERD_ALLOCATED
 serd_node_malloc(size_t n_bytes, SerdNodeFlags flags, SerdType type);
 
 void
-serd_node_set(SerdNode** dst, const SerdNode* src);
+serd_node_set(SerdNode* SERD_NULLABLE* SERD_NONNULL dst,
+              const SerdNode* SERD_NULLABLE         src);
 
-#endif // SERD_NODE_H
+#endif // SERD_SRC_NODE_H
